config_require(target/snmpTargetAddrEntry)
config_require(target/snmpTargetParamsEntry)
config_require(target/target)
config_version_require((target/target_counters, 5.5, target/target_counters_5_5))
config_add_mib(SNMP-TARGET-MIB)
